<?php

/**
 * @file
 * Provides page callback for COMMENT ABUSE module.
 */

/**
 * Callback function for abuse comment.
 */
function comment_abuse_content($cid, $data = array()) {
  // Create ajax commands.
  $commands = array();

  $abuse_result = FALSE;
  if (!empty($cid)) {
    // Add new abuse.
    $abuse_result = comment_abuse_add_comment_abuse($cid, $data);
  }

  if ($abuse_result) {
    // Translate message.
    $message = t('Your complaint has been received.');

    // Replace abuse link with recieved message.
    $commands[] = ajax_command_replace(
      '.comment-abuse-' . $cid,
      '<span class="abuse-notice">' . check_plain($message) . '</span>'
    );
  }
  else {
    // Translate message.
    $message = t('Your complaint has been not received.');

    // Replace abuse link with recieved message.
    $commands[] = ajax_command_replace(
      '.comment-abuse-' . $cid,
      '<span class="abuse-notice">' . check_plain($message) . '</span>'
    );
  }

  // Complaint form mode.
  if (variable_get('comment_abuse_use_popup', 1)) {
    $commands[] = ajax_command_replace(
      '#complaint-form-wrapper',
      theme(
        'comment_abuse_complaint_popup_result',
        array(
          'message' => check_plain($message),
          'result' => $abuse_result,
        )
      )
    );

    ajax_deliver(array('#type' => 'ajax', '#commands' => $commands));
  }
  // Single link mode.
  else {
    return array('#type' => 'ajax', '#commands' => $commands);
  }
}

/**
 * Return popup with form for complaint on comment.
 */
function comment_abuse_complaint_popup($cid) {
  // POST is empty - load popup.
  if (empty($_POST)) {
    $form = drupal_render(drupal_get_form('comment_abuse_complaint_form'));
    print theme('comment_abuse_complaint_popup', array('form' => $form));
    exit();
  }
  // If we have POST - process fields values.
  else {
    // Validation.
    $errors = comment_abuse_complaint_form_validate($_POST);
    // Show errors.
    if ($errors) {
      $commands = array();
      $commands[] = ajax_command_replace(
        '#comment_abuse_popup_messages',
        $errors
      );
      ajax_deliver(array('#type' => 'ajax', '#commands' => $commands));
    }
    // Add complaint.
    else {
      module_load_include('inc', 'comment_abuse', 'comment_abuse.pages');
      comment_abuse_content($cid, $_POST);
    }
  }
}
